"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getProofTypesSupported = void 0;
exports.getTypesFromCredentialSupported = getTypesFromCredentialSupported;
exports.credentialConfigurationSupportedToCredentialSupported = credentialConfigurationSupportedToCredentialSupported;
exports.credentialSupportedToCredentialConfigurationSupported = credentialSupportedToCredentialConfigurationSupported;
exports.credentialsSupportedV13ToV11 = credentialsSupportedV13ToV11;
exports.credentialsSupportedV11ToV13 = credentialsSupportedV11ToV13;
exports.getOfferedCredentials = getOfferedCredentials;
const core_1 = require("@credo-ts/core");
const utils_1 = require("./utils");
/**
 * Get all `types` from a `CredentialSupported` object.
 *
 * Depending on the format, the types may be nested, or have different a different name/type
 */
function getTypesFromCredentialSupported(credentialSupported) {
    if (credentialSupported.format === 'jwt_vc_json-ld' ||
        credentialSupported.format === 'ldp_vc' ||
        credentialSupported.format === 'jwt_vc_json' ||
        credentialSupported.format === 'jwt_vc') {
        return credentialSupported.credential_definition.type;
    }
    else if (credentialSupported.format === 'vc+sd-jwt') {
        return credentialSupported.vct;
    }
    throw Error(`Unable to extract types from credentials supported. Unknown format ${credentialSupported.format}`);
}
function credentialConfigurationSupportedToCredentialSupported(id, config) {
    var _a, _b;
    const baseConfig = {
        id,
        scope: config.scope,
        cryptographic_binding_methods_supported: config.cryptographic_binding_methods_supported,
        cryptographic_suites_supported: config.credential_signing_alg_values_supported,
        // In theory credentials_supported should not have any proof_types we do this to allow back and forth conversion
        proof_types_supported: config.proof_types_supported,
        display: config.display,
        order: config.order,
    };
    if (config.format === 'jwt_vc_json' || config.format === 'jwt_vc') {
        return Object.assign(Object.assign({}, baseConfig), { format: config.format, credentialSubject: config.credential_definition.credentialSubject, types: (_a = config.credential_definition.type) !== null && _a !== void 0 ? _a : [] });
    }
    else if (config.format === 'ldp_vc' || config.format === 'jwt_vc_json-ld') {
        return Object.assign(Object.assign({}, baseConfig), { format: config.format, 
            // @ts-expect-error this should exist
            '@context': config.credential_definition['@context'], credentialSubject: config.credential_definition.credentialSubject, types: (_b = config.credential_definition.type) !== null && _b !== void 0 ? _b : [] });
    }
    else if (config.format === 'vc+sd-jwt') {
        return Object.assign(Object.assign({}, baseConfig), { format: config.format, 
            // vct must be defined when format is vc+sd-jwt
            vct: config.vct, claims: config.claims });
    }
    throw new core_1.CredoError(`Unsupported credential format ${config.format}`);
}
function credentialSupportedToCredentialConfigurationSupported(credentialSupported, options) {
    var _a;
    const baseCredentialConfigurationSupported = {
        id: credentialSupported.id,
        scope: credentialSupported.scope,
        cryptographic_binding_methods_supported: credentialSupported.cryptographic_binding_methods_supported,
        credential_signing_alg_values_supported: 'cryptographic_suites_supported' in credentialSupported
            ? credentialSupported.cryptographic_suites_supported
            : undefined,
        // In theory credentials_supported should not have any proof_types we do this to allow back and forth conversion
        proof_types_supported: (_a = credentialSupported.proof_types_supported) !== null && _a !== void 0 ? _a : options === null || options === void 0 ? void 0 : options.proofTypesSupported,
        display: credentialSupported.display,
        order: credentialSupported.order,
    };
    if (credentialSupported.format === 'jwt_vc_json' || credentialSupported.format === 'jwt_vc') {
        return Object.assign(Object.assign({}, baseCredentialConfigurationSupported), { format: credentialSupported.format, credential_definition: {
                credentialSubject: credentialSupported.credentialSubject,
                type: credentialSupported.types,
            } });
    }
    else if (credentialSupported.format === 'ldp_vc' || credentialSupported.format === 'jwt_vc_json-ld') {
        return Object.assign(Object.assign({}, baseCredentialConfigurationSupported), { format: credentialSupported.format, credential_definition: {
                // @ts-expect-error this should exist
                '@context': credentialSupported['@context'],
                credentialSubject: credentialSupported.credentialSubject,
                type: credentialSupported.types,
            } });
    }
    else if (credentialSupported.format === 'vc+sd-jwt') {
        return Object.assign(Object.assign({}, baseCredentialConfigurationSupported), { format: credentialSupported.format, vct: credentialSupported.vct, id: credentialSupported.id, claims: credentialSupported.format === 'vc+sd-jwt' ? credentialSupported.claims : undefined, credential_definition: {} });
    }
    throw new core_1.CredoError(`Unsupported credential format ${credentialSupported.format}`);
}
function credentialsSupportedV13ToV11(credentialConfigurationSupported) {
    const credentialsSupportedWithId = [];
    for (const [id, credentialConfiguration] of Object.entries(credentialConfigurationSupported)) {
        credentialsSupportedWithId.push(credentialConfigurationSupportedToCredentialSupported(id, credentialConfiguration));
    }
    return credentialsSupportedWithId;
}
function credentialsSupportedV11ToV13(credentialsSupported, options) {
    const credentialConfigurationsSupported = {};
    for (const credentialSupported of credentialsSupported) {
        credentialConfigurationsSupported[credentialSupported.id] = credentialSupportedToCredentialConfigurationSupported(credentialSupported, options);
    }
    return credentialConfigurationsSupported;
}
/**
 * Returns all entries from the credential offer with the associated metadata resolved. For 'id' entries, the associated `credentials_supported` object is resolved from the issuer metadata.
 * For inline entries, an error is thrown.
 */
function getOfferedCredentials(offeredCredentials, allCredentialsSupported, options) {
    const credentialConfigurationsOffered = {};
    const uniformCredentialsSupported = Array.isArray(allCredentialsSupported)
        ? credentialsSupportedV11ToV13(allCredentialsSupported, options)
        : allCredentialsSupported;
    for (const offeredCredential of offeredCredentials) {
        // In draft 12 inline credential offers are removed. It's easier to already remove support now.
        if (typeof offeredCredential !== 'string') {
            throw new core_1.CredoError('Only referenced credentials pointing to an id in credentials_supported issuer metadata are supported');
        }
        // Make sure the issuer metadata includes the offered credential.
        const credentialConfigurationSupported = uniformCredentialsSupported[offeredCredential];
        if (!credentialConfigurationSupported) {
            throw new Error(`Offered credential '${offeredCredential}' is not part of credentials_supported of the issuer metadata.`);
        }
        credentialConfigurationsOffered[offeredCredential] = credentialConfigurationSupported;
    }
    return credentialConfigurationsOffered;
}
const getProofTypesSupported = (agentContext) => {
    return {
        jwt: {
            proof_signing_alg_values_supported: (0, utils_1.getSupportedJwaSignatureAlgorithms)(agentContext),
        },
        cwt: {
            proof_signing_alg_values_supported: [],
        },
        ldp_vp: {
            proof_signing_alg_values_supported: [],
        },
    };
};
exports.getProofTypesSupported = getProofTypesSupported;
//# sourceMappingURL=issuerMetadataUtils.js.map